@@ -359,6 +359,12 @@ int prompt(bool verbose, CompilerOptions &cu)
                 std::cout << std::setprecision(17) << "(" << r.c64.re << ", " << r.c64.im << ")" << std::endl;
                 break;
             }
+            case (LCompilers::FortranEvaluator::EvalResult::boolean) : {
+                if (verbose) std::cout << "Return type: logical" << std::endl;
+                if (verbose) section("Result:");
+                std::cout << (r.b ? "True" : "False") << std::endl;
+                break;
+            }
             case (LCompilers::FortranEvaluator::EvalResult::statement) : {
                 if (verbose) {
                     std::cout << "Return type: none" << std::endl;
@@ -1561,15 +1567,35 @@ int link_executable(const std::vector<std::string> &infiles,
             compile_cmd += runtime_library_dir + "\\lfortran_runtime_static.lib";
             run_cmd = outfile;
         } else if (LCompilers::startswith(t, "wasm")) {
-            char* wasi_sdk_path = std::getenv("WASI_SDK_PATH");
-            if (wasi_sdk_path == nullptr) {
-                std::cerr << "WASI_SDK_PATH must be defined to use llvm->wasm\n";
-                return 11;
+            std::string CC, options, runtime_lib;
+            if (LCompilers::endswith(t, "wasi")) {
+                char* wasi_sdk_path = std::getenv("WASI_SDK_PATH");
+                if (wasi_sdk_path == nullptr) {
+                    std::cerr << "WASI_SDK_PATH must be defined to use llvm->wasm\n";
+                    return 11;
+                }
+                CC = std::string(wasi_sdk_path) + "/bin/clang";
+                options = " --target=wasm32-wasi -nostartfiles -Wl,--entry=_start -Wl,-lwasi-emulated-process-clocks";
+                runtime_lib = "lfortran_runtime_wasm_wasi.o";
+                compile_cmd = CC + options + " -o " + outfile + " ";
+            } else if (LCompilers::endswith(t, "emscripten")) {
+                char* emsdk_path = std::getenv("EMSDK_PATH");
+                if (emsdk_path == nullptr) {
+                    std::cerr << "EMSDK_PATH must be defined to use llvm->wasm\n";
+                    return 11;
+                }
+                CC = std::string(emsdk_path) + "/upstream/emscripten/emcc";
+                options = " --target=wasm32-unknown-emscripten -sSTACK_SIZE=50mb -sINITIAL_MEMORY=256mb";
+                if (!compiler_options.emcc_embed.empty()) {
+                    options += " --embed-file " + compiler_options.emcc_embed;
+                }
+                runtime_lib = "lfortran_runtime_wasm_emcc.o";
+                compile_cmd = CC + options + " -o " + outfile +
+                     (compiler_options.wasm_html ? ".html " : " ");
+            } else {
+                std::cerr << "Unsupported target: " << t << std::endl;
+                return 10;
             }
-            std::string CC = std::string(wasi_sdk_path) + "/bin/clang";
-            std::string options = " --target=wasm32-wasi -nostartfiles -Wl,--entry=_start -Wl,-lwasi-emulated-process-clocks";
-            std::string runtime_lib = "lfortran_runtime_wasm.o";
-            compile_cmd = CC + options + " -o " + outfile + " ";
             for (auto &s : infiles) {
                 compile_cmd += s + " ";
             }
@@ -1617,6 +1643,13 @@ int link_executable(const std::vector<std::string> &infiles,
                 compile_cmd += extra_linker_flags;
             }
             compile_cmd += " -l" + runtime_lib + " -lm";
+            if (compiler_options.openmp) {
+                std::string openmp_shared_library = compiler_options.openmp_lib_dir;
+                std::string omp_cmd =  " -L" + openmp_shared_library + " -Wl,-rpath," + openmp_shared_library + " -lomp";
+                if (!openmp_shared_library.empty()) {
+                    compile_cmd += omp_cmd;
+                }
+            }
             run_cmd = "./" + outfile;
         }
         if (verbose) {
@@ -1757,7 +1790,12 @@ int link_executable(const std::vector<std::string> &infiles,
     } else if (t == "x86_64-pc-windows-msvc") {
         run_cmd = outfile;
     } else if (LCompilers::startswith(t, "wasm")) {
-        run_cmd = "wasmtime " + outfile + " --dir=.";
+        if (LCompilers::endswith(t, "wasi")) {
+            run_cmd = "wasmtime " + outfile + " --dir=.";
+        } else if (LCompilers::endswith(t, "emscripten")) {
+            run_cmd = "node " + outfile +
+                (compiler_options.wasm_html ? ".js" : "");
+        }
     } else {
         run_cmd = "./" + outfile;
     }
@@ -1930,6 +1968,7 @@ int main_app(int argc, char *argv[]) {
     std::vector<std::string> arg_l;
     std::vector<std::string> arg_L;
     std::vector<std::string> arg_files;
+    std::string arg_standard;
     bool arg_version = false;
     bool show_prescan = false;
     bool show_tokens = false;
@@ -2010,7 +2049,8 @@ int main_app(int argc, char *argv[]) {
     app.add_flag("--no-cpp", no_cpp, "Disable C preprocessing");
     app.add_flag("--fixed-form", compiler_options.fixed_form, "Use fixed form Fortran source parsing");
     app.add_flag("--fixed-form-infer", fixed_form_infer, "Use heuristics to infer if a file is in fixed form");
-    app.add_flag("--no-prescan", arg_no_prescan, "Turn off prescan");
+    app.add_option("--std", arg_standard, "Select standard conformance (lf, f23, legacy)");
+    app.add_flag("--no-prescan", arg_no_prescan, "Turn off prescanning");
     app.add_flag("--show-prescan", show_prescan, "Show tokens for the given file and exit");
     app.add_flag("--show-tokens", show_tokens, "Show tokens for the given file and exit");
     app.add_flag("--show-ast", show_ast, "Show AST for the given file and exit");
@@ -2037,11 +2077,14 @@ int main_app(int argc, char *argv[]) {
     app.add_flag("--time-report", time_report, "Show compilation time report");
     app.add_flag("--static", static_link, "Create a static executable");
     app.add_flag("--shared", shared_link, "Create a shared executable");
+    app.add_flag("--logical-casting", compiler_options.logical_casting, "Allow logical casting");
     app.add_flag("--no-warnings", compiler_options.no_warnings, "Turn off all warnings");
+    app.add_flag("--no-style-warnings", compiler_options.disable_style, "Turn off style suggestions");
     app.add_flag("--no-error-banner", compiler_options.no_error_banner, "Turn off error banner");
     app.add_option("--error-format", compiler_options.error_format, "Control how errors are produced (human, short)")->capture_default_str();
     app.add_option("--backend", arg_backend, "Select a backend (llvm, c, cpp, x86, wasm, fortran)")->capture_default_str();
     app.add_flag("--openmp", compiler_options.openmp, "Enable openmp");
+    app.add_flag("--openmp-lib-dir", compiler_options.openmp_lib_dir, "Pass path to openmp library")->capture_default_str();
     app.add_flag("--generate-object-code", compiler_options.generate_object_code, "Generate object code into .o files");
     app.add_flag("--rtlib", compiler_options.rtlib, "Include the full runtime library in the LLVM output");
     app.add_flag("--use-loop-variable-after-loop", compiler_options.po.use_loop_variable_after_loop, "Allow using loop variable after the loop");
@@ -2069,6 +2112,8 @@ int main_app(int argc, char *argv[]) {
     app.add_flag("--legacy-array-sections", compiler_options.legacy_array_sections, "Enables passing array items as sections if required");
     app.add_flag("--ignore-pragma", compiler_options.ignore_pragma, "Ignores all the pragmas");
     app.add_flag("--stack-arrays", compiler_options.stack_arrays, "Allocate memory for arrays on stack");
+    app.add_flag("--wasm-html", compiler_options.wasm_html, "Generate HTML file using emscripten for LLVM->WASM");
+    app.add_option("--emcc-embed", compiler_options.emcc_embed, "Embed a given file/directory using emscripten for LLVM->WASM");
 
     /*
     * Subcommands:
@@ -2124,9 +2169,39 @@ int main_app(int argc, char *argv[]) {
 #endif
     }
 
+    if (arg_standard == "" || arg_standard == "lf") {
+        // The default LFortran behavior, do nothing
+    } else if (arg_standard == "f23") {
+        compiler_options.disable_style = true;
+        compiler_options.implicit_typing = true;
+        compiler_options.implicit_argument_casting = true;
+        compiler_options.implicit_interface = true;
+        compiler_options.print_leading_space = true;
+        compiler_options.logical_casting = false;
+    } else if (arg_standard == "legacy") {
+        // f23
+        compiler_options.disable_style = true;
+        compiler_options.implicit_typing = true;
+        compiler_options.implicit_argument_casting = true;
+        compiler_options.implicit_interface = true;
+        compiler_options.print_leading_space = true;
+        compiler_options.logical_casting = false;
+
+        // legacy options
+        compiler_options.fixed_form = true;
+        compiler_options.legacy_array_sections = true;
+        compiler_options.use_loop_variable_after_loop = true;
+        compiler_options.generate_object_code = true;
+    } else {
+        std::cerr << "The option `--std=" << arg_standard << "` is not supported" << std::endl;
+        return 1;
+    }
+
     compiler_options.use_colors = !arg_no_color;
     compiler_options.indent = !arg_no_indent;
     compiler_options.prescan = !arg_no_prescan;
+    // set openmp in pass options
+    compiler_options.po.openmp = compiler_options.openmp;
 
     for (auto &f_flag : f_flags) {
         if (f_flag == "PIC") {
