@@ -14,7 +14,7 @@ namespace LCompilers {
                 llvm::FunctionType *function_type = llvm::FunctionType::get(
                         llvm::Type::getInt8PtrTy(context), {
                             llvm::Type::getInt32Ty(context)
-                        }, true);
+                        }, false);
                 fn = llvm::Function::Create(function_type,
                         llvm::Function::ExternalLinkage, func_name, module);
             }
@@ -31,7 +31,7 @@ namespace LCompilers {
                         llvm::Type::getInt8PtrTy(context), {
                             llvm::Type::getInt8PtrTy(context),
                             llvm::Type::getInt32Ty(context)
-                        }, true);
+                        }, false);
                 fn = llvm::Function::Create(function_type,
                         llvm::Function::ExternalLinkage, func_name, module);
             }
@@ -221,6 +221,15 @@ namespace LCompilers {
             return LLVM::CreateLoad(*builder, dim_des_arr_ptr);
         }
 
+        llvm::Value* SimpleCMODescriptor::
+        get_pointer_to_dimension_descriptor_array(llvm::Type* type, llvm::Value* arr, bool load) {
+            llvm::Value* dim_des_arr_ptr = llvm_utils->create_gep2(type, arr, 2);
+            if( !load ) {
+                return dim_des_arr_ptr;
+            }
+            return LLVM::CreateLoad(*builder, dim_des_arr_ptr);
+        }
+
         llvm::Value* SimpleCMODescriptor::
         get_rank(llvm::Value* arr, bool get_pointer) {
             llvm::Value* rank_ptr = llvm_utils->create_gep(arr, 4);
@@ -620,7 +629,7 @@ namespace LCompilers {
             return idx;
         }
 
-        llvm::Value* SimpleCMODescriptor::get_single_element(llvm::Value* array,
+        llvm::Value* SimpleCMODescriptor::get_single_element(llvm::Type *type, llvm::Value* array,
             std::vector<llvm::Value*>& m_args, int n_args, bool data_only,
             bool is_fixed_size, llvm::Value** llvm_diminfo, bool polymorphic,
             llvm::Type* polymorphic_type, bool is_unbounded_pointer_to_data) {
@@ -633,9 +642,9 @@ namespace LCompilers {
                 LCOMPILERS_ASSERT(llvm_diminfo);
                 idx = cmo_convertor_single_element_data_only(llvm_diminfo, m_args, n_args, check_for_bounds, is_unbounded_pointer_to_data);
                 if( is_fixed_size ) {
-                    tmp = llvm_utils->create_gep(array, idx);
+                    tmp = llvm_utils->create_gep2(type, array, idx);
                 } else {
-                    tmp = llvm_utils->create_ptr_gep(array, idx);
+                    tmp = llvm_utils->create_ptr_gep2(type, array, idx);
                 }
             } else {
                 idx = cmo_convertor_single_element(array, m_args, n_args, check_for_bounds);
@@ -772,7 +781,7 @@ namespace LCompilers {
                 llvm::Value* dim_size_ptr = llvm_utils->create_gep(dim_val, 2);
                 builder->CreateStore(llvm::ConstantInt::get(context, llvm::APInt(32, 1)), l_val);
                 builder->CreateStore(LLVM::CreateLoad(*builder, prod), s_val);
-                llvm::Value* dim_size = LLVM::CreateLoad(*builder, llvm_utils->create_ptr_gep(shape_data, r_val));
+                llvm::Value* dim_size = builder->CreateSExtOrTrunc(LLVM::CreateLoad(*builder, llvm_utils->create_ptr_gep(shape_data, r_val)), llvm::Type::getInt32Ty(context));
                 builder->CreateStore(builder->CreateMul(LLVM::CreateLoad(*builder, prod), dim_size), prod);
                 builder->CreateStore(dim_size, dim_size_ptr);
                 r_val = builder->CreateAdd(r_val, llvm::ConstantInt::get(context, llvm::APInt(32, 1)));
@@ -874,7 +883,7 @@ namespace LCompilers {
             uint64_t size = data_layout.getTypeAllocSize(llvm_data_type);
             llvm::Value* llvm_size = llvm::ConstantInt::get(context, llvm::APInt(32, size));
             num_elements = builder->CreateMul(num_elements, llvm_size);
-            builder->CreateMemCpy(dest, llvm::MaybeAlign(), src, llvm::MaybeAlign(), num_elements);
+            builder->CreateMemCpy(src, llvm::MaybeAlign(), dest, llvm::MaybeAlign(), num_elements);
         }
 
     } // LLVMArrUtils
