@@ -794,7 +794,13 @@ intrinsic_funcs_args = {
            "return": "int32",
        },
    ],
-
+    "Real": [
+       {
+           "args": [("int",), ("real",), ("complex",)],
+           "return": "real32",
+           "kind_arg": True,
+       },
+   ],
 }
 
 skip_create_func = ["Partition"]
