@@ -10,6 +10,54 @@
 
 namespace LCompilers {
 
+
+class ReplaceArrayPhysicalCast: public ASR::BaseExprReplacer<ReplaceArrayPhysicalCast> {
+    private:
+        Allocator& al;
+    public:
+        std::vector<std::string> array_variables;
+
+        ReplaceArrayPhysicalCast(Allocator& al_) :
+            al(al_) {}
+
+        void replace_ArrayPhysicalCast(ASR::ArrayPhysicalCast_t* x) {
+            ASRUtils::ASRBuilder b(al, x->base.base.loc);
+            ASR::symbol_t* sym = ASR::down_cast<ASR::Var_t>(x->m_arg)->m_v;
+            std::string sym_name = ASRUtils::symbol_name(sym);
+            if (std::find(array_variables.begin(), array_variables.end(), sym_name) != array_variables.end()) {
+                *current_expr = b.Var(sym);
+            }
+        }
+};
+
+class ArrayPhysicalCastVisitor : public ASR::CallReplacerOnExpressionsVisitor<ArrayPhysicalCastVisitor> {
+    private:
+        std::string function_name;
+        ReplaceArrayPhysicalCast replacer;
+        std::vector<std::string> &array_variables;
+    public:
+        ArrayPhysicalCastVisitor(Allocator &al_, std::vector<std::string> &array_variables_, std::string function_name_) :
+            function_name(function_name_), replacer(al_), array_variables(array_variables_) {}
+
+        void call_replacer() {
+            replacer.array_variables = array_variables;
+            replacer.current_expr = current_expr;
+            replacer.replace_expr(*current_expr);
+        }
+
+        void visit_SubroutineCall(const ASR::SubroutineCall_t &x) {
+            if (ASRUtils::symbol_name(x.m_name) == function_name) {
+                CallReplacerOnExpressionsVisitor::visit_SubroutineCall(x);
+            }
+        }
+
+        void visit_FunctionCall(const ASR::FunctionCall_t &x) {
+            if (ASRUtils::symbol_name(x.m_name) == function_name) {
+                CallReplacerOnExpressionsVisitor::visit_FunctionCall(x);
+            }
+        }
+};
+
 class ReplaceArrayVariable: public ASR::BaseExprReplacer<ReplaceArrayVariable> {
     private:
         Allocator& al;
@@ -31,6 +79,19 @@ class ReplaceArrayVariable: public ASR::BaseExprReplacer<ReplaceArrayVariable> {
                 *current_expr = b.Var(sym);
             }
         }
+
+        void replace_ArrayPhysicalCast(ASR::ArrayPhysicalCast_t* x) {
+            ASRUtils::ASRBuilder b(al, x->base.base.loc);
+            if (ASR::is_a<ASR::Var_t>(*x->m_arg)) {
+                ASR::Var_t* var = ASR::down_cast<ASR::Var_t>(x->m_arg);
+                if (std::find(array_variables.begin(), array_variables.end(), ASRUtils::symbol_name(var->m_v)) != array_variables.end() &&
+                    ASRUtils::symbol_parent_symtab(var->m_v)->counter == current_scope->counter) {
+                    ASR::symbol_t* sym = current_scope->get_symbol(std::string(ASRUtils::symbol_name(var->m_v)));
+                    LCOMPILERS_ASSERT(sym != nullptr);
+                    *current_expr = b.Var(sym);
+                }
+            }
+        }
 };
 
 class ArrayVisitor: public ASR::CallReplacerOnExpressionsVisitor<ArrayVisitor> {
@@ -120,13 +181,13 @@ class FunctionSubroutineCallVisitor: public ASR::BaseWalkVisitor<FunctionSubrout
         std::vector<SymbolTable*> &scopes;
         std::vector<int> &array_variable_indices;
         std::vector<std::string> &array_variables;
-        std::map<int, std::map<std::string, ASR::symbol_t*>> &scoped_array_variable_map;
+        std::map<int, std::map<std::string, std::vector<ASR::symbol_t*>>> &scoped_array_variable_map;
     
     public:
         FunctionSubroutineCallVisitor(std::string function_name_, std::vector<SymbolTable*> &scopes_,
             std::vector<int> &array_variable_indices_,
             std::vector<std::string> &array_variables_,
-            std::map<int, std::map<std::string, ASR::symbol_t*>> &scoped_array_variable_map_) :
+            std::map<int, std::map<std::string, std::vector<ASR::symbol_t*>>> &scoped_array_variable_map_) :
             function_name(function_name_), scopes(scopes_), array_variable_indices(array_variable_indices_),
             array_variables(array_variables_),
             scoped_array_variable_map(scoped_array_variable_map_) {}
@@ -155,7 +216,7 @@ class FunctionSubroutineCallVisitor: public ASR::BaseWalkVisitor<FunctionSubrout
                     for (size_t i = 0; i < array_variable_indices.size(); i++) {
                         ASR::symbol_t* sym = current_scope->get_symbol(array_variables[i]);
                         LCOMPILERS_ASSERT(sym != nullptr);
-                        scoped_array_variable_map[current_scope->counter][array_variables[i]] = sym;
+                        scoped_array_variable_map[current_scope->counter][array_variables[i]].push_back(sym);
                     }
                 }
             }
@@ -177,7 +238,7 @@ class FunctionSubroutineCallVisitor: public ASR::BaseWalkVisitor<FunctionSubrout
                         ASR::Var_t* var = ASR::down_cast<ASR::Var_t>(arg);
                         ASR::symbol_t* sym = current_scope->get_symbol(ASRUtils::symbol_name(var->m_v));
                         LCOMPILERS_ASSERT(sym != nullptr);
-                        scoped_array_variable_map[current_scope->counter][array_variables[i]] = sym;
+                        scoped_array_variable_map[current_scope->counter][array_variables[i]].push_back(sym);
                     }
                 }
             }
@@ -197,7 +258,7 @@ class FunctionSubroutineCallVisitor: public ASR::BaseWalkVisitor<FunctionSubrout
                         ASR::Var_t* var = ASR::down_cast<ASR::Var_t>(arg);
                         ASR::symbol_t* sym = current_scope->get_symbol(ASRUtils::symbol_name(var->m_v));
                         LCOMPILERS_ASSERT(sym != nullptr);
-                        scoped_array_variable_map[current_scope->counter][array_variables[i]] = sym;
+                        scoped_array_variable_map[current_scope->counter][array_variables[i]].push_back(sym);
                     }
                 }
             }
@@ -271,12 +332,13 @@ class ReplaceExpression: public ASR::BaseExprReplacer<ReplaceExpression> {
 
 class DoConcurrentStatementVisitor : public ASR::CallReplacerOnExpressionsVisitor<DoConcurrentStatementVisitor> {
     private:
+        Allocator& al;
         SymbolTable* current_scope;
         ReplaceExpression replacer;
 
     public:
         DoConcurrentStatementVisitor(Allocator &al_, SymbolTable* current_scope_) :
-            current_scope(current_scope_), replacer(al_) {}
+            al(al_), current_scope(current_scope_), replacer(al_) {}
 
     void call_replacer() {
         replacer.current_expr = current_expr;
@@ -287,6 +349,18 @@ class DoConcurrentStatementVisitor : public ASR::CallReplacerOnExpressionsVisito
     void visit_FunctionCall(const ASR::FunctionCall_t &x) {
         ASR::FunctionCall_t* x_copy = const_cast<ASR::FunctionCall_t*>(&x);
         ASR::symbol_t* func_sym = current_scope->get_symbol(ASRUtils::symbol_name(x.m_name));
+        if (func_sym == nullptr) {
+            // this means we have a user defined function and need to create an interface for it
+            ASR::Function_t* func = ASR::down_cast<ASR::Function_t>(x.m_name);
+            ASRUtils::SymbolDuplicator duplicator(al);
+            // This will create trouble when we have something like x(n) inside function
+            // duplicator does not duplicate type of variables inside function
+            func_sym = duplicator.duplicate_Function(func, current_scope);
+            current_scope->add_symbol(ASRUtils::symbol_name(func_sym), func_sym);
+            ASR::Function_t* new_func = ASR::down_cast<ASR::Function_t>(func_sym);
+            new_func->m_body = nullptr; new_func->n_body = 0;
+            ASR::down_cast<ASR::FunctionType_t>(new_func->m_function_signature)->m_deftype = ASR::deftypeType::Interface;
+        }
         LCOMPILERS_ASSERT(func_sym != nullptr);
         x_copy->m_name = func_sym;
         x_copy->m_original_name = func_sym;
@@ -316,6 +390,7 @@ class DoConcurrentVisitor :
         Allocator& al;
         bool remove_original_statement;
         Vec<ASR::stmt_t*> pass_result;
+        Vec<ASR::stmt_t*> pass_result_allocatable;
         SymbolTable* current_scope;
         PassOptions pass_options;
         int current_stmt_index = -1;
@@ -325,9 +400,43 @@ class DoConcurrentVisitor :
         DoConcurrentVisitor(Allocator& al_, PassOptions pass_options_) :
         al(al_), remove_original_statement(false), pass_options(pass_options_) {
             pass_result.n = 0;
+            pass_result_allocatable.n = 0;
         }
 
         void transform_stmts(ASR::stmt_t **&m_body, size_t &n_body) {
+            bool remove_original_statement_copy = remove_original_statement;
+            Vec<ASR::stmt_t*> body;
+            body.reserve(al, n_body);
+            current_m_body = m_body;
+            current_n_body = n_body;
+            for (size_t i=0; i<n_body; i++) {
+                pass_result.n = 0;
+                pass_result_allocatable.n = 0;
+                current_stmt_index = i;
+                pass_result.reserve(al, 1);
+                pass_result_allocatable.reserve(al, 1);
+                remove_original_statement = false;
+                visit_stmt(*m_body[i]);
+                for (size_t j=0; j < pass_result.size(); j++) {
+                    body.push_back(al, pass_result[j]);
+                }
+                for (size_t j=0; j < pass_result_allocatable.size(); j++) {
+                    body.push_front(al, pass_result_allocatable[j]);
+                }
+                if( !remove_original_statement ) {
+                    body.push_back(al, m_body[i]);
+                }
+            }
+            m_body = body.p;
+            n_body = body.size();
+            current_m_body = nullptr;
+            current_n_body = 0;
+            pass_result.n = 0;
+            pass_result_allocatable.n = 0;
+            remove_original_statement = remove_original_statement_copy;
+        }
+
+        void transform_stmts_do_loop(ASR::stmt_t **&m_body, size_t &n_body) {
             bool remove_original_statement_copy = remove_original_statement;
             Vec<ASR::stmt_t*> body;
             body.reserve(al, n_body);
@@ -526,7 +635,7 @@ class DoConcurrentVisitor :
                 b.VariableDeclaration(current_scope, it.first, sym_type, ASR::intentType::Local);
                 if (is_array) {
                     // add lbound and ubound variables for array
-                    ASR::Array_t* arr_type = ASR::down_cast<ASR::Array_t>(it.second);
+                    ASR::Array_t* arr_type = ASR::down_cast<ASR::Array_t>(ASRUtils::type_get_past_allocatable(ASRUtils::type_get_past_pointer(it.second)));
                     for (size_t i = 0; i < arr_type->n_dims; i++) {
                         std::string lbound_name = "lbound_" + it.first + "_" + std::to_string(i);
                         std::string ubound_name = "ubound_" + it.first + "_" + std::to_string(i);
@@ -554,23 +663,31 @@ class DoConcurrentVisitor :
             return {thread_data_module_name, thread_data_struct};
         }
 
+        std::vector<ASR::symbol_t*> create_modules_for_lcompilers_function(const Location &loc) {
+            std::vector<ASR::symbol_t*> module_symbols;
+            ASR::symbol_t* mod_sym = create_module(loc, "iso_c_binding");
+            LCOMPILERS_ASSERT(mod_sym != nullptr && ASR::is_a<ASR::Module_t>(*mod_sym));
+            module_symbols.push_back(mod_sym);
+            mod_sym = create_module(loc, "omp_lib");
+            LCOMPILERS_ASSERT(mod_sym != nullptr && ASR::is_a<ASR::Module_t>(*mod_sym));
+            module_symbols.push_back(mod_sym);
+            return module_symbols;
+        }
+
         ASR::symbol_t* create_lcompilers_function(const Location &loc, const ASR::DoConcurrentLoop_t &do_loop,
-                    std::map<std::string, ASR::ttype_t*> &involved_symbols, std::string thread_data_module_name) {
+                    std::map<std::string, ASR::ttype_t*> &involved_symbols, std::string thread_data_module_name,
+                    std::vector<ASR::symbol_t*> module_symbols) {
             SymbolTable* current_scope_copy = current_scope;
             while (current_scope->parent != nullptr) {
                 current_scope = current_scope->parent;
             }
             current_scope = al.make_new<SymbolTable>(current_scope);
             // load modules
-            ASR::symbol_t* mod_sym = create_module(loc, "iso_c_binding");
-            LCOMPILERS_ASSERT(mod_sym != nullptr && ASR::is_a<ASR::Module_t>(*mod_sym));
-            std::string unsupported_sym_name = import_all(ASR::down_cast<ASR::Module_t>(mod_sym));
+            std::string unsupported_sym_name = import_all(ASR::down_cast<ASR::Module_t>(module_symbols[0]));
             LCOMPILERS_ASSERT(unsupported_sym_name == "");
-            mod_sym = create_module(loc, "omp_lib");
-            LCOMPILERS_ASSERT(mod_sym != nullptr && ASR::is_a<ASR::Module_t>(*mod_sym));
-            unsupported_sym_name = import_all(ASR::down_cast<ASR::Module_t>(mod_sym));
+            unsupported_sym_name = import_all(ASR::down_cast<ASR::Module_t>(module_symbols[1]));
             LCOMPILERS_ASSERT(unsupported_sym_name == "");
-            mod_sym = create_module(loc, thread_data_module_name);
+            ASR::symbol_t* mod_sym = create_module(loc, thread_data_module_name);
             LCOMPILERS_ASSERT(mod_sym != nullptr && ASR::is_a<ASR::Module_t>(*mod_sym));
             unsupported_sym_name = import_all(ASR::down_cast<ASR::Module_t>(mod_sym));
             LCOMPILERS_ASSERT(unsupported_sym_name == "");
@@ -791,8 +908,23 @@ class DoConcurrentVisitor :
                         body.push_back(al, b.Assignment(lhs, b.Mul(lhs, red.m_arg)));
                         break;
                     }
+                    case ASR::reduction_opType::ReduceMAX : {
+                        body.push_back(al, b.If(b.Lt(lhs, red.m_arg), {
+                            b.Assignment(lhs, red.m_arg)
+                        }, {
+                            // do nothing
+                        }));
+                        break;
+                    }
+                    case ASR::reduction_opType::ReduceMIN : {
+                        body.push_back(al, b.If(b.Gt(lhs, red.m_arg), {
+                            b.Assignment(lhs, red.m_arg)
+                        }, {
+                            // do nothing
+                        }));
+                        break;
+                    }
                     default : {
-                        // TODO: support ReduceMAX, ReduceMIN
                         LCOMPILERS_ASSERT(false);
                     }
                 }
@@ -849,7 +981,7 @@ class DoConcurrentVisitor :
             4. Recursively call this function for all the function calls found in step 3
         */
         void recursive_function_call_resolver(SymbolTable* current_scope, std::vector<std::string> array_variables,
-                std::map<int, std::map<std::string, ASR::symbol_t*>> scoped_array_variable_map, bool first_call=false) {
+                std::map<int, std::map<std::string, std::vector<ASR::symbol_t*>>> scoped_array_variable_map, bool first_call=false, std::string func_name = "") {
             ASR::asr_t* asr_owner = current_scope->asr_owner;
             if (ASR::is_a<ASR::symbol_t>(*asr_owner)) {
                 ASR::symbol_t* sym = ASR::down_cast<ASR::symbol_t>(asr_owner);
@@ -862,33 +994,26 @@ class DoConcurrentVisitor :
                         for (size_t i = 0; i < array_variables.size(); i++) {
                             ASR::symbol_t* sym = current_scope->resolve_symbol(array_variables[i]);
                             ASR::ttype_t* sym_type = ASRUtils::symbol_type(sym);
-                            if (ASR::is_a<ASR::Pointer_t>(*sym_type)) {
-                                continue;
-                            } else if (ASR::is_a<ASR::Array_t>(*sym_type)) {
+                            // look at comment in Program_t case
+                            if (ASR::is_a<ASR::Array_t>(*sym_type)) {
                                 ASR::Array_t* array_type = ASR::down_cast<ASR::Array_t>(sym_type);
-                                if (!ASRUtils::is_dimension_empty(*array_type->m_dims) || is_interface) {
-                                    Vec<ASR::dimension_t> dims; dims.reserve(al, array_type->n_dims);
-                                    ASR::dimension_t empty_dim; empty_dim.loc = array_type->base.base.loc;
-                                    empty_dim.m_start = nullptr; empty_dim.m_length = nullptr;
-                                    for (size_t i = 0; i < array_type->n_dims; i++) {
-                                        dims.push_back(al, empty_dim);
-                                    }
-                                    ASR::expr_t* array_expr = b.VariableOverwrite(current_scope, ASRUtils::symbol_name(sym),
-                                            ASRUtils::TYPE(ASR::make_Pointer_t(al, array_type->base.base.loc,
-                                                    ASRUtils::TYPE(ASR::make_Array_t(al, array_type->base.base.loc,
-                                                    array_type->m_type, dims.p, dims.n, ASR::array_physical_typeType::DescriptorArray)))),
-                                                ASR::intentType::InOut);
-                                    LCOMPILERS_ASSERT(array_expr != nullptr);
-                                    /*
-                                        We allocate memory for array variables only if we have information about their sizes.
-                                    */
-                                    if (!is_interface) new_body.push_back(al, b.Allocate(array_expr, array_type->m_dims, array_type->n_dims));
-                                } else {
-                                    // we have no information about what size to allocate
+                                bool dimension_empty = ASRUtils::is_dimension_empty(*array_type->m_dims);
+                                Vec<ASR::dimension_t> dims; dims.reserve(al, array_type->n_dims);
+                                ASR::dimension_t empty_dim; empty_dim.loc = array_type->base.base.loc;
+                                empty_dim.m_start = nullptr; empty_dim.m_length = nullptr;
+                                for (size_t i = 0; i < array_type->n_dims; i++) {
+                                    dims.push_back(al, empty_dim);
                                 }
-                            } else if (ASR::is_a<ASR::Allocatable_t>(*sym_type)) {
-                                // TODO: handle allocatable
-                                continue;
+                                ASR::expr_t* array_expr = b.VariableOverwrite(current_scope, ASRUtils::symbol_name(sym),
+                                        ASRUtils::TYPE(ASR::make_Pointer_t(al, array_type->base.base.loc,
+                                                ASRUtils::TYPE(ASR::make_Array_t(al, array_type->base.base.loc,
+                                                array_type->m_type, dims.p, dims.n, ASR::array_physical_typeType::DescriptorArray)))),
+                                            check_is_argument(current_scope, ASRUtils::symbol_name(sym)) ? ASR::intentType::InOut : ASR::intentType::Local);
+                                LCOMPILERS_ASSERT(array_expr != nullptr);
+                                /*
+                                    We allocate memory for array variables only if we have information about their sizes.
+                                */
+                                if (!is_interface && !dimension_empty) new_body.push_back(al, b.Allocate(array_expr, array_type->m_dims, array_type->n_dims));
                             }
                         }
                         for (size_t i = 0; i < func->n_body; i++) {
@@ -944,9 +1069,11 @@ class DoConcurrentVisitor :
                         if (it->counter != current_scope->counter) {
                             std::vector<std::string> new_array_variables;
                             for (auto it2: scoped_array_variable_map[it->counter]) {
-                                new_array_variables.push_back(ASRUtils::symbol_name(it2.second));
+                                for (auto it3: it2.second) {
+                                    new_array_variables.push_back(ASRUtils::symbol_name(it3));
+                                }
                             }
-                            recursive_function_call_resolver(it, new_array_variables, scoped_array_variable_map);
+                            recursive_function_call_resolver(it, new_array_variables, scoped_array_variable_map, false, func->m_name);
                         }
                     }
                     scopes.clear();
@@ -958,10 +1085,18 @@ class DoConcurrentVisitor :
                         for (size_t i = 0; i < array_variables.size(); i++) {
                             ASR::symbol_t* sym = current_scope->get_symbol(array_variables[i]);
                             ASR::ttype_t* sym_type = ASRUtils::symbol_type(sym);
-                            if (ASR::is_a<ASR::Pointer_t>(*sym_type)) {
-                                continue;
-                            } else if (ASR::is_a<ASR::Array_t>(*sym_type)) {
+                            /*
+                                For pointer and allocatable arrays, we already have the symbol with the correct type
+                                and we don't need to allocate memory for them as they are already allocated.
+
+                                So special handling is required only for non-pointer and non-allocatable arrays.
+                            */
+                            if (ASR::is_a<ASR::Array_t>(*sym_type)) {
                                 ASR::Array_t* array_type = ASR::down_cast<ASR::Array_t>(sym_type);
+                                /*
+                                    More precisely dimension cannot be empty for arrays in program, so
+                                    it is just a check to see if we have information about the size of the array.
+                                */
                                 if (!ASRUtils::is_dimension_empty(*array_type->m_dims)) {
                                     Vec<ASR::dimension_t> dims; dims.reserve(al, array_type->n_dims);
                                     ASR::dimension_t empty_dim; empty_dim.loc = array_type->base.base.loc;
@@ -979,9 +1114,6 @@ class DoConcurrentVisitor :
                                 } else {
                                     // we have no information about what size to allocate
                                 }
-                            } else if (ASR::is_a<ASR::Allocatable_t>(*sym_type)) {
-                                // TODO: handle allocatable
-                                continue;
                             }
                         }
                         for (size_t i = 0; i < prog->n_body; i++) {
@@ -994,20 +1126,9 @@ class DoConcurrentVisitor :
                     ArrayVisitor v(al, prog->m_symtab, array_variables);
                     for (size_t i=0; i<prog->n_body; i++) {
                         // we can create a replacer but then there will be too many replacers to handle.
-                        if (ASR::is_a<ASR::SubroutineCall_t>(*prog->m_body[i])) {
-                            ASR::SubroutineCall_t* sub_call = ASR::down_cast<ASR::SubroutineCall_t>(prog->m_body[i]);
-                            for (size_t j = 0; j < sub_call->n_args; j++) {
-                                ASR::expr_t* arg = sub_call->m_args[j].m_value;
-                                if (ASR::is_a<ASR::ArrayPhysicalCast_t>(*arg)) {
-                                    ASR::symbol_t* sym = ASR::down_cast<ASR::Var_t>(ASR::down_cast<ASR::ArrayPhysicalCast_t>(arg)->m_arg)->m_v;
-                                    std::string sym_name = ASRUtils::symbol_name(sym);
-                                    if (std::find(array_variables.begin(), array_variables.end(), sym_name) != array_variables.end()) {
-                                        sub_call->m_args[j].m_value = b.Var(sym);
-                                    }
-                                }
-                            }
-                        }
-                        // TODO: handle function calls
+                        ArrayPhysicalCastVisitor apcv(al, array_variables, func_name);
+                        apcv.current_expr = nullptr;
+                        apcv.visit_stmt(*prog->m_body[i]);
                         v.visit_stmt(*prog->m_body[i]);
                     }
                 }
@@ -1016,6 +1137,19 @@ class DoConcurrentVisitor :
             }
         }
 
+        bool check_is_argument(SymbolTable *current_scope, std::string arg_name) {
+            if (ASR::is_a<ASR::symbol_t>(*current_scope->asr_owner) &&
+                ASR::is_a<ASR::Function_t>(*ASR::down_cast<ASR::symbol_t>(current_scope->asr_owner)) ) {
+                ASR::Function_t* func = ASR::down_cast<ASR::Function_t>(ASR::down_cast<ASR::symbol_t>(current_scope->asr_owner));
+                for (size_t i = 0; i < func->n_args; i++) {
+                    if (ASRUtils::symbol_name(ASR::down_cast<ASR::Var_t>(func->m_args[i])->m_v) == arg_name) {
+                        return true;
+                    }
+                }
+            }
+            return false;
+        }
+
         void visit_DoConcurrentLoop(const ASR::DoConcurrentLoop_t &x) {
             std::map<std::string, ASR::ttype_t*> involved_symbols;
 
@@ -1024,6 +1158,7 @@ class DoConcurrentVisitor :
 
             // create thread data module
             std::pair<std::string, ASR::symbol_t*> thread_data_module = create_thread_data_module(involved_symbols, x.base.base.loc);
+            std::vector<ASR::symbol_t*> module_symbols = create_modules_for_lcompilers_function(x.base.base.loc);
 
             // create external symbol for the thread data module
             ASR::symbol_t* thread_data_ext_sym = ASR::down_cast<ASR::symbol_t>(ASR::make_ExternalSymbol_t(al, x.base.base.loc,
@@ -1034,20 +1169,24 @@ class DoConcurrentVisitor :
             std::vector<std::string> array_variables;
             // create data variable for the thread data module
             ASRUtils::ASRBuilder b(al, x.base.base.loc);
-            ASR::expr_t* data_expr = b.Variable(current_scope, "data", ASRUtils::TYPE(ASR::make_StructType_t(al, x.base.base.loc, thread_data_ext_sym)), ASR::intentType::Local);
+            ASR::expr_t* data_expr = b.Variable(current_scope, current_scope->get_unique_name("data"), ASRUtils::TYPE(ASR::make_StructType_t(al, x.base.base.loc, thread_data_ext_sym)), ASR::intentType::Local);
             LCOMPILERS_ASSERT(data_expr != nullptr);
 
             // now create a tdata (cptr)
-            ASR::expr_t* tdata_expr = b.Variable(current_scope, "tdata", ASRUtils::TYPE(ASR::make_CPtr_t(al, x.base.base.loc)), ASR::intentType::Local);
+            ASR::expr_t* tdata_expr = b.Variable(current_scope, current_scope->get_unique_name("tdata"), ASRUtils::TYPE(ASR::make_CPtr_t(al, x.base.base.loc)), ASR::intentType::Local);
             LCOMPILERS_ASSERT(tdata_expr != nullptr);
 
             // TODO: update symbols with correct type
             for (auto it: involved_symbols) {
                 ASR::ttype_t* sym_type = it.second;
                 if (ASR::is_a<ASR::Pointer_t>(*sym_type)) {
+                    // everything is already handled
+                    array_variables.push_back(it.first);
                     continue;
-                } else if (ASR::is_a<ASR::Array_t>(*sym_type)) {
-                    ASR::Array_t* array_type = ASR::down_cast<ASR::Array_t>(sym_type);
+                } else if (ASR::is_a<ASR::Array_t>(*ASRUtils::type_get_past_allocatable(sym_type))) {
+                    bool is_argument = check_is_argument(current_scope, it.first);
+                    bool is_allocatable = ASR::is_a<ASR::Allocatable_t>(*sym_type);
+                    ASR::Array_t* array_type = ASR::down_cast<ASR::Array_t>(ASRUtils::type_get_past_allocatable(sym_type));
                     Vec<ASR::dimension_t> dims; dims.reserve(al, array_type->n_dims);
                     ASR::dimension_t empty_dim; empty_dim.loc = array_type->base.base.loc;
                     empty_dim.m_start = nullptr; empty_dim.m_length = nullptr;
@@ -1058,7 +1197,7 @@ class DoConcurrentVisitor :
                             ASRUtils::TYPE(ASR::make_Pointer_t(al, array_type->base.base.loc,
                                     ASRUtils::TYPE(ASR::make_Array_t(al, array_type->base.base.loc,
                                     array_type->m_type, dims.p, dims.n, ASR::array_physical_typeType::DescriptorArray)))),
-                                ASR::intentType::InOut);
+                                is_argument ? ASR::intentType::InOut : ASR::intentType::Local);
                     LCOMPILERS_ASSERT(array_expr != nullptr);
                     bool already_allocated = true;
                     if (ASR::is_a<ASR::symbol_t>(*current_scope->asr_owner) && ASR::is_a<ASR::Function_t>(*ASR::down_cast<ASR::symbol_t>(current_scope->asr_owner))) {
@@ -1094,14 +1233,16 @@ class DoConcurrentVisitor :
                         Reason to comment this line can be found in function `recursive_function_call_resolver`
                     */
                     // if (!already_allocated) pass_result.push_back(al, b.Allocate(array_expr, array_type->m_dims, array_type->n_dims));
+                    /*
+                        If it is not an argument, we need to allocate memory for it.
+                        But if it is also an allocatable, we assume that user will allocate memory for it or code is incorrect.
+                    */
+                    if (!is_argument && !is_allocatable) pass_result_allocatable.push_back(al, b.Allocate(array_expr, array_type->m_dims, array_type->n_dims));
                     involved_symbols[it.first] = ASRUtils::expr_type(array_expr);
                     array_variables.push_back(it.first);
-                } else if (ASR::is_a<ASR::Allocatable_t>(*sym_type)) {
-                    // TODO: handle allocatable
-                    continue;
                 }
             }
-            
+
             // add external symbols to struct members, we need those for `data%n = n`
             ASR::symbol_t* thread_data_sym = thread_data_module.second;
             SymbolTable* thread_data_symtab = ASRUtils::symbol_symtab(thread_data_sym);
@@ -1161,7 +1302,7 @@ class DoConcurrentVisitor :
                     ASRUtils::expr_type(tdata_expr), nullptr))
             ));
 
-            ASR::symbol_t* lcompilers_function = create_lcompilers_function(x.base.base.loc, x, involved_symbols, thread_data_module.first);
+            ASR::symbol_t* lcompilers_function = create_lcompilers_function(x.base.base.loc, x, involved_symbols, thread_data_module.first, module_symbols);
             LCOMPILERS_ASSERT(lcompilers_function != nullptr);
             ASR::Function_t* lcompilers_func = ASR::down_cast<ASR::Function_t>(lcompilers_function);
             ASR::symbol_t* lcompilers_interface_function = create_interface_lcompilers_function(lcompilers_func);
@@ -1192,17 +1333,25 @@ class DoConcurrentVisitor :
             pass_result.push_back(al, ASRUtils::STMT(ASR::make_SubroutineCall_t(al, x.base.base.loc, current_scope->get_symbol("gomp_parallel"), nullptr,
                                 call_args.p, call_args.n, nullptr)));
 
-            // update all reduction variables in statements after this.
-            ReductionVariableVisitor rvv(al, current_scope, ASRUtils::symbol_name(thread_data_sym), data_expr, reduction_variables);
-            for (size_t i = current_stmt_index + 1; i < current_n_body; i++) {
-                rvv.visit_stmt(*current_m_body[i]);
+            for (auto it: reduction_variables) {
+                ASR::symbol_t* actual_sym = current_scope->resolve_symbol(it);
+                ASR::symbol_t* sym = current_scope->get_symbol(std::string(ASRUtils::symbol_name(thread_data_sym)) + "_" + it);
+                LCOMPILERS_ASSERT(sym != nullptr);
+                pass_result.push_back(al, b.Assignment(
+                    b.Var(actual_sym),
+                    ASRUtils::EXPR(ASR::make_StructInstanceMember_t(al, x.base.base.loc, data_expr, sym, ASRUtils::symbol_type(sym), nullptr)
+                )));
             }
             reduction_variables.clear();
 
             if (array_variables.size() > 0) {
                 // std::vector<std::string> function_names; function_names.push_back(ASRUtils::symbol_name(ASR::down_cast<ASR::symbol_t>(current_scope->asr_owner)));
-                std::map<int, std::map<std::string, ASR::symbol_t*>> scoped_array_variable_map;
-                recursive_function_call_resolver(current_scope, array_variables, scoped_array_variable_map, true);
+                std::map<int, std::map<std::string, std::vector<ASR::symbol_t*>>> scoped_array_variable_map;
+                std::string func_name = "";
+                if (ASR::is_a<ASR::symbol_t>(*current_scope->asr_owner)) {
+                    func_name = ASRUtils::symbol_name(ASR::down_cast<ASR::symbol_t>(current_scope->asr_owner));
+                }
+                recursive_function_call_resolver(current_scope, array_variables, scoped_array_variable_map, true, func_name);
             }
 
             remove_original_statement = true;
@@ -1237,6 +1386,15 @@ class DoConcurrentVisitor :
             current_scope = current_scope_copy;
         }
 
+        void visit_DoLoop(const ASR::DoLoop_t &x) {
+            ASR::DoLoop_t& xx = const_cast<ASR::DoLoop_t&>(x);
+
+            visit_do_loop_head(xx.m_head);
+
+            transform_stmts_do_loop(xx.m_body, xx.n_body);
+            transform_stmts_do_loop(xx.m_orelse, xx.n_orelse);
+        }
+
 };
 
 void pass_replace_openmp(Allocator &al, ASR::TranslationUnit_t &unit,
